#ifndef FrameCPP_VERSION_6_FrTOCAdcData_HH
#define FrameCPP_VERSION_6_FrTOCAdcData_HH

#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Common
  {
    class TOCInfo;
  }

  namespace Version_6
  {
    //===================================================================
    //===================================================================
    class FrTOCAdcData
    {
    public:
      typedef INT_4U	nadc_type;
      typedef STRING	name_type;
      typedef INT_4U	channel_id_type;
      typedef INT_4U	group_id_type;
      typedef INT_8U	position_type;

      struct adc_info_type {
	//---------------------------------------------------------------
	/// \brief Channel ID of ADC
	//---------------------------------------------------------------
	channel_id_type			m_channelID;
	//---------------------------------------------------------------
	/// \brief Group ID of ADC
	//---------------------------------------------------------------
	group_id_type			m_groupID;
	//---------------------------------------------------------------
	/// \brief FrAdcData offset positions.
	///
	/// FrAdcData offset positions, in bytes,
	/// from the beginning of the file.
	/// The size of the array is nFrame
	//---------------------------------------------------------------
	std::vector< position_type >	m_positionADC;
      };

      typedef std::vector< name_type >			key_container_type;
      typedef std::map< name_type, adc_info_type >	MapADC_type;

      FrTOCAdcData( );

      Common::FrameSpec::size_type
      Bytes( const Common::StreamBase& Stream ) const;

      const MapADC_type& GetADC( ) const;

      MapADC_type::const_iterator GetADC( const std::string& Name ) const;

      MapADC_type::const_iterator GetADC( INT_4U index ) const;

      //-----------------------------------------------------------------
      /// \brief Retrieve the keys
      ///
      /// \return
      ///     Constant container of keys.
      //-----------------------------------------------------------------
      const key_container_type& GetKeys( ) const;

      void QueryAdc( const Common::TOCInfo& Info,
		     INT_4U FrameOffset,
		     INT_8U Position );

      bool operator==( const FrTOCAdcData& RHS ) const;

      template< typename SE >
      static void Description( Common::Description& Desc );

    protected:
      key_container_type	m_keys;
      MapADC_type		m_info;

      FrTOCAdcData( Common::IStream& Stream, INT_4U FrameCount );
      void write( Common::OStream& Stream ) const;
    };

    inline Common::FrameSpec::size_type FrTOCAdcData::
    Bytes( const Common::StreamBase& Stream ) const
    {
      Common::FrameSpec::size_type 
	retval = sizeof( nadc_type );
      if ( m_info.size( ) )
      {
	retval
	  += ( m_info.size( )
	       * ( sizeof( channel_id_type )
		   + sizeof( group_id_type )
		   + ( sizeof( position_type )
		       * m_info.begin( )->second.m_positionADC.size( ) )
		   ) );
      }
      for ( MapADC_type::const_iterator
	      cur = m_info.begin( ),
	      last = m_info.end( );
	    cur != last;
	    ++cur )
      {
	retval += cur->first.Bytes( );
      }
      return retval;
    }

    template< typename SE >
    void FrTOCAdcData::
    Description( Common::Description& Desc )
    {
      Desc( SE( "nADC", "INT_4U",
		"Number of unique FrAdcData names in file." ) );
      Desc( SE( "name", "*STRING", "Array of FrAdcData names" ) );
      Desc( SE( "channelID", "*INT_4U",
		"Array of ADC channel IDs" ) );
      Desc( SE( "groupID", "*INT_4U",
		"Array of ADC group IDs" ) );
      Desc( SE( "positionADC", "*INT_8U",
		"Array of lists of FrAdcData offset positions, in bytes,"
		" from beginning of file (size of nFrame*nADC)"
		) );
    }

    inline const FrTOCAdcData::MapADC_type& FrTOCAdcData::
    GetADC( ) const
    {
      return m_info;
    }

    inline const FrTOCAdcData::key_container_type& FrTOCAdcData::
    GetKeys( ) const
    {
      return m_keys;
    }

  } // namespace - Version_6
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_6_FrTOCAdcData_HH */
